require "rd/rd2html-lib"
require 'rd/rdfmt'
require 'amrita2/ext'

module Amrita2
  include Amrita2
  include Amrita2::HtmlTag
  TMPL = REXML::Document.new <<-END
  <html>
  <head id='header'>
     <title id='title' />
     <link id='css' rel="stylesheet" type="text/css" />
  </head>
  <body id='document'>
    <div id='output'></div>
    <div id='block'>
      <h1 id='header1'> header</h1>
      <h2 id='header2'> header</h2>
      <h3 id='header3'> header</h3>
      <p id='textblock'> </p>
      <ul id='list'><li id='item'></li></ul>
      <ol id='enumlist'><li id='item'></li></ol>
      <dl id='desclist'><dt id='term' /><dd id='description' /></dl>
    </div>
    <div id='inline'>
      <em id='emphasis'>*<span id='text'></span>*</em>
      <code id='code'><span id='text'></span></code>
      <var id='var'><span id='text'></span></var>
      <kbd id='kbd'><span id='text'></span></kbd>
      <a id='link_to_url'></a>
    </div>
  </body>
  </html>
  END

  SPEC = Amrita2::define_template_spec do 
    dynamic_element(:header, :delete_id=>true) do 
      dynamic_element(:title) 
      dynamic_element(:css, :replace_attr=>true) 
    end
    dynamic_element(:document,  :delete_id=>true) do
      dynamic_element(:output,  :delete_tag=>true) 
      dynamic_element(:block,  :delete_tag=>true) do
        parts_element(:header1)
        parts_element(:header2)
        parts_element(:header3)
        parts_element(:textblock) 
        parts_element(:list) do 
          dynamic_element(:item) 
        end
        parts_element(:enumlist) do 
          dynamic_element(:item) 
        end
        parts_element(:desclist) do 
          dynamic_element(:term) 
          dynamic_element(:description, :output_stream=>:term) 
        end
      end
      dynamic_element(:inline) do
        dynamic_element(:emphasis) do 
          dynamic_element(:text, :delete_tag=>true, :matcher=>SPAN)
        end
        dynamic_element(:code) do 
          dynamic_element(:text, :delete_tag=>true, :matcher=>SPAN)
        end
        dynamic_element(:var) do 
          dynamic_element(:text, :delete_tag=>true, :matcher=>SPAN)
        end
        dynamic_element(:kbd) do 
          dynamic_element(:text, :delete_tag=>true, :matcher=>SPAN)
        end
        dynamic_element(:link_to_url, :use_original_element=>true)
      end
    end
  end

  RDTmpl = SPEC.create_template_module(TMPL)

  class Amrita2Visitor < RD::RD2HTMLVisitor
    attr_accessor :option
    def initialize(doctmpl, blocktmpl, inlinetmpl, extention_processor, option=nil)
      @doctmpl, @blocktmpl, @inlinetmpl, @extention_processor =
        doctmpl, blocktmpl, inlinetmpl, extention_processor
      extend @blocktmpl
      extend @inlinetmpl
      extend @doctmpl
      @option = option
    end

    def visit_DocumentElement(element)
      expand_template(out="") do |m|
        if @option
          m.header do |mh|
            mh.title(@option[:title]) if @option[:title]
            mh.css(nil,:href=>@option[:css]) if @option[:css]
          end
        end
        m.document do |md|
          md.output(visit_children(element))
        end
      end
      out
    end

    def visit_children(element)
      SanitizedString[super.to_s]
    end

    def visit_TextBlock(element)
      if (is_this_textblock_only_one_block_of_parent_listitem?(element) or
	  is_this_textblock_only_one_block_other_than_sublists_in_parent_listitem?(element))
        visit_children(element).gsub(/\s/, " ")
      else
        textblock(visit_children(element))
      end
    end
    HeadlineMethods = [
      nil,
      :header1,
      :header2,
      :header3,
      :header4,
      :header5,
      :header6,
      nil
    ]

    def visit_Headline(element)
      c = visit_children(element)
      self.send(HeadlineMethods[element.level], c)
    end

    def visit_ItemList(element)
      list do |m|
        visit_children(element)
      end
    end

    def visit_ItemListItem(element)
      @blocktmpl::List::item(visit_children(element))
    end

    def visit_EnumList(element)
      enumlist do
        visit_children(element)
      end
    end

    def apply_to_EnumListItem(element, content)
      @blocktmpl::Enumlist::item(content)
    end

    def visit_DescList(element)
      desclist do
        visit_children(element)
      end
    end

    def apply_to_DescListItem(element, term, description)
      @blocktmpl::Desclist::term(term)
      @blocktmpl::Desclist::description(description)
    end

    def visit_StringElement(element)
      element.to_label.amrita_sanitize
    end

    def visit_Emphasis(element)
      @inlinetmpl::emphasis do |m|
        m.text(visit_children(element))
      end
    end

    def visit_Code(element)
      @inlinetmpl::code do |m|
        m.text(visit_children(element))
      end
    end

    def visit_Var(element)
      @inlinetmpl::var do |m|
        m.text(visit_children(element))
      end
    end

    def visit_Keyboard(element)
      @inlinetmpl::kbd do |m|
        m.text(visit_children(element))
      end
    end

    def visit_Reference(element)
      case element.label
      when RD::Reference::RDLabel
        name, *a = element.to_label.split(':')
        process_ext(name,  *a)
      when  RD::Reference::URL
        new_context do
          @inlinetmpl::link_to_url do |e|
            e.attributes.delete('id')
            e.attributes['href'] = element.label.url
            e.text = visit_children(element)
            e 
          end
          output_substream(:link_to_url)
        end
      else
        raise "can't happen"
      end
    end
    
    def apply_to_MethodListItem(element, term, content)
      name, *args =  term.split
      @extention_processor.send(name.intern, *args) { content } 
    end

    def apply_to_MethodList(element, items)
      items
    end

    def process_ext(name, *args)
      @extention_processor.send(name.intern, *args)
    end
  end

  class RDAStandardExtention
    include Amrita2::Runtime
    include Amrita2::HtmlTag
    TMPL = REXML::Document.new <<-END
    <div id='extention'>
      <a id='rubytalk' href="http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/$1">[ruby-talk:$1]</a>
      <a id='raa' href="http://raa.ruby-lang.org/list.rhtml?name=$1">[$2]</a>]
      <table id='table1' border='1'>
         <tr><th id='header_item'></th></tr>
         <tr class='a'><th id='header_item'></th><td id='detail_item'></td></tr>
         <tr class='b'><th id='header_item'></th><td id='detail_item'></td></tr>
      </table>
      <blockquote id='blockquote' cite='$1'><p>$0</p><p>(<a href='$1'>$1</a>)</p></blockquote>
      <div id='include'><pre>$1$0</pre></div>
    </div>
    END

    SPEC = Amrita2::define_template_spec do 
      dynamic_element(:extention, :delete_id=>true) do
        parts_element(:rubytalk, :use_args=>true)
        parts_element(:raa, :use_args=>true)
        parts_element(:table1) do 
          dynamic_element(:header, :matcher=>TR) do
            dynamic_element(:header_item, :matcher=>TH)
          end
          dynamic_element(:detail, :matcher=>TR, :match_many=>true) do
            dynamic_element(:header_item, :matcher=>TH)
            dynamic_element(:item, :matcher=>TD)
          end
        end
        parts_element(:blockquote, :use_args=>true)
        parts_element(:include, :use_args=>true)
      end
    end
    include SPEC.create_template_module(TMPL)
    public :expand_template

    def table(name, cols, *args)
      content = yield
      cols = cols.to_i
      items = content.split
      ret = self.send(name) do |mt|
        mt.header do |mh|
          cols.times do |i|
            mh.header_item(items.shift) 
          end
        end
        lines = items.size/cols
        lines.times do |i|
          mt.detail(i) do |md|
            md.header_item(items.shift)
            (cols-1).times do |j|
              md.item(items.shift)
            end
          end
        end
      end
      ret
    end

    def include(name, title=name)
      super("--- #{title} ---\n") { File::open(name).read }
    end
  end

  class RDAProcessor

    def self.compile_rd(path, out=nil, opts={})
      out = path + ".html" unless out
      css = opts[:css] 
      File::open(path) do |i|
        File::open(out, "w") do |out|
          src = i.read
          ret = Amrita2::RDAProcessor.new(opts).process(src, opts)
          out.print ret
        end
      end
    end

    def initialize(opts={})
      doc = opts[:doc_template] || RDTmpl
      block = opts[:block_template] || RDTmpl::MainContext::Document::Block
      inline = opts[:inline_template] || RDTmpl::MainContext::Document::Inline
      ext = opts[:extention_processor] || RDAStandardExtention.new
      @visitor = Amrita2Visitor.new(doc, block, inline, ext)
    end

    def process(rdtext, opt=nil)
      tree = RD::RDTree.new("=begin\n#{rdtext}\n=end\n\n")
      @visitor.option = opt
      @visitor.visit(tree)
    end
  end
end
