

require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/template'
require 'amrita2/ext'
require 'rexml/document'
require 'open3'

# ruby -I~/amrita2/lib ~/amrita2/test/test_template.rb   |ruby -pe '$_.sub!(/^ */, "") ' 

module Amrita2TemplateTest

  class TestDefaultSpec < Test::Unit::TestCase
    include Amrita2
    
    def test_default_spec
      doc = REXML::Document.new "<a id='aaa'><b id='bbb'></b></a>"
      root = doc.root
      spec = Amrita2::make_default_spec(root)
      assert_equal(nil, spec.sym)
      assert_equal(1, spec.children.size)
      assert_equal(:aaa, spec.children[0].sym)
      assert_equal(1, spec.children[0].children.size)
      assert_equal(:bbb, spec.children[0].children[0].sym)
      assert_equal(0, spec.children[0].children[0].children.size)

      t = spec.create_template(doc)

      root = doc.root
      assert(root.amrita_spec)
      assert(:aaa, root.amrita_spec.sym)
      assert(root.elements["b"].amrita_spec)
      assert(:aaa, root.elements["b"].amrita_spec.sym)

      out = ""
      t.expand_template(out) do |m|
        m.aaa do |m2|
          m2.bbb('yyy')
        end
      end

      assert_equal("<a><b>yyy</b></a>", out)
    end

    def test_default_spec2
      doc = REXML::Document.new <<-END
<html>
   <body>
      <h1 id='title'>title will be inserted here</h1>
      <p id='body'>body text will be inserted here</p>
   </body>
</html>
END

      root = doc.root
      spec = Amrita2::make_default_spec(root)
      assert_equal(nil, spec.sym)
      assert_equal(2, spec.children.size)
      assert_equal(:title, spec.children[0].sym)
      assert_equal(0, spec.children[0].children.size)
      assert_equal(:body, spec.children[1].sym)

      t = spec.create_template(doc)

      root = doc.root
      assert(root.amrita_spec)
      assert(:aaa, root.amrita_spec.sym)
      assert(root.elements["body/h1"].amrita_spec)
      assert(:aaa, root.elements["body/h1"].amrita_spec.sym)

      out = ""
      t.expand_template(out) do |m|
        m.title('ttt')
        m.body('bbb')
      end

      assert_equal("<html>\n   <body>\n      <h1>ttt</h1>\n      <p>bbb</p>\n   </body>\n</html>", out)
    end
  end

  class TestExpander < Test::Unit::TestCase
    include Amrita2
    class A
      def aaa
        B.new
      end

      def struct_array
        [B.new, B.new]
      end
    end

    class B
      def bbb
        'xxx'
      end
      
      def array
        [1,2,3]
      end

      def hash
        {:a=>'b', :c=>'d'}
      end

      def hasharray
        [ hash, hash]
      end
    end

    def test_expander
      t = TemplateText.new "<a id='aaa'><b id='bbb'></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b>xxx</b></a>", out)
    end

    def test_array
      t = TemplateText.new "<a id='aaa'><b id='array'></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b>1</b><b>2</b><b>3</b></a>", out)
    end

    def test_struct_array
      t = TemplateText.new "<a id='struct_array'><b id='bbb'></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b>xxx</b></a><a><b>xxx</b></a>", out)
    end


    def test_hash
      t = TemplateText.new "<a id='aaa'><b id='hash'><c id='a'></c></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b><c>b</c></b></a>", out)
    end

    def test_hasharray
      t = TemplateText.new "<a id='aaa'><b id='hasharray'><c id='a'></c></b></a>" 
      t.expand(out="", A.new) 
      assert_equal("<a><b><c>b</c></b><b><c>b</c></b></a>", out)
    end

  end

  class TestDefaultExpand < Test::Unit::TestCase
    include Amrita2

    def test_default_spec
      doc = REXML::Document.new "<a id='aaa'><b id='bbb'></b></a>"
      root = doc.root
      spec = Amrita2::make_default_spec(root)

      t = spec.create_template(doc)

      root = doc.root
      assert(root.amrita_spec)
      assert(:aaa, root.amrita_spec.sym)
      assert(root.elements["b"].amrita_spec)
      assert(:aaa, root.elements["b"].amrita_spec.sym)

      data = {
        :aaa =>  {
          :bbb => "yyy"
        }
      }
      data = {
        'aaa' =>  {
          'bbb' => "yyy"
        }
      }
      out = ""
      Expander.new.expand(t, out, data)
      assert_equal("<a><b>yyy</b></a>", out)

      data = {
        :aaa =>  {
          :bbb => [1,2,3]
        }
      }
      out = ""
      Expander.new.expand(t, out, data)
      assert_equal("<a><b>1</b><b>2</b><b>3</b></a>", out)
    end
  end

  class TestTemplate < Test::Unit::TestCase
    include Amrita2

    def test_template
      t = TemplateText.new "<a id='aaa'><b id='bbb'></b></a>" 
      t.expand(out="") do |m|
        m.aaa do |m2|
          m2.bbb('xxx')
        end
      end
      assert_equal("<a><b>xxx</b></a>", out)
    end

    def test_spec
      text = "<a id='aaa'><b id='bbb'></b></a>" 
      t = TemplateText.new(text) 
      t.spec =Amrita2::define_template_spec do
        dynamic_element(:aaa) do 
          dynamic_element(:bbb, :use_original_element=>true)
        end
      end
      t.expand(out="") do |m|
        m.aaa do |m2|
          m2.bbb do |e|
            e.attributes['class'] = 'using_spec'
            e.text = 'using spec'
            e
          end
        end
      end
      assert_equal("<a id='aaa'><b class='using_spec' id='bbb'>using spec</b></a>", out)
    end

    def test_spec2
      text = "<a id='aaa'><b id='bbb'></b></a>" 
      t = TemplateText.new(text)  do
        Amrita2::define_template_spec do
          dynamic_element(:aaa) do 
            dynamic_element(:bbb, :use_original_element=>true)
          end
        end
      end
      t.expand(out="") do |m|
        m.aaa do |m2|
          m2.bbb do |e|
            e.attributes['class'] = 'using_spec2'
            e.text = 'using spec2'
            e
          end
        end
      end
      assert_equal("<a id='aaa'><b class='using_spec2' id='bbb'>using spec2</b></a>", out)
    end

    def test_spec3
      text = "<a id='aaa'><b id='bbb'></b></a>" 
      t = TemplateText.new(text)
      def t.get_spec
        Amrita2::define_template_spec do
          dynamic_element(:aaa) do 
            dynamic_element(:bbb, :use_original_element=>true)
          end
        end
      end
      t.expand(out="") do |m|
        m.aaa do |m2|
          m2.bbb do |e|
            e.attributes['class'] = 'using_spec3'
            e.text = 'using spec3'
            e
          end
        end
      end
      assert_equal("<a id='aaa'><b class='using_spec3' id='bbb'>using spec3</b></a>", out)
    end

    def test_spec4
      text = "<x id='aaa'><y id='bbb'>1</y><y id='bbb'>2</y></x>"
      t = TemplateText.new(text) do
        Amrita2::define_template_spec do
          dynamic_element(:aaa) do
            dynamic_element(:bbb) 
            dummy_element(:bbb) # 222 
          end
        end
      end

      data = {"aaa"=>{:bbb=>["111", "222"]}}
      t.expand(out="", data)
      assert_equal("<x id='aaa'><y id='bbb'>111</y><y id='bbb'>222</y></x>", out)
    end

    class TestPO
      class Child
        def bbb
          'xxx'
        end
      end
      def aaa
        Child.new
      end

      def active(m)
        m.active do |m|
          m.bbb('xxx')
          m.bbb('yyy')
          m.bbb('zzz')
        end
      end

      def attr(m)
        m.attr(:aaa=>'bbb') do |m|
          m.bbb('xxx', :yyy=>'zzz')
        end
      end
    end

    def test_presentatio_object
      t = TemplateText.new "<a id='aaa'><b id='bbb'></b></a>" 
      t.expand(out="", TestPO.new) 
      assert_equal("<a><b>xxx</b></a>", out)

      t = TemplateText.new "<a id='active'><b id='bbb'></b></a>" 
      t.expand(out="", TestPO.new) 
      assert_equal("<a><b>xxx</b><b>yyy</b><b>zzz</b></a>", out)


      t = TemplateText.new "<x id='outer'><a id='active'><b id='bbb'></b></a></x>" 
      t.expand(out="", :outer=>TestPO.new) 
      assert_equal("<x><a><b>xxx</b><b>yyy</b><b>zzz</b></a></x>", out)

      t = TemplateText.new "<x id='outer'><a id='active'><b id='bbb'></b></a></x>" 
      t.expand(out="", :outer=>[TestPO.new, TestPO.new]) 
      assert_equal("<x><a><b>xxx</b><b>yyy</b><b>zzz</b></a></x><x><a><b>xxx</b><b>yyy</b><b>zzz</b></a></x>", out)

      t = TemplateText.new "<x id='outer'><a id='attr'><b id='bbb'></b></a></x>" 
      t.expand(out="", 'outer'=>TestPO.new) 
      assert_equal("<x><a aaa='bbb'><b yyy='zzz'>xxx</b></a></x>", out)
    end

    Lang = Struct.new(:name, :author, :website)
    class LangClass
      attr_reader :name, :author
      def initialize(n, a)
        @name, @author = n, a
      end
    end

    def test_template2
      tmpl = TemplateText.new <<-END
        <table>
          <tr><th>name</th><th>author</th></tr>
          <tr id='table1'>
            <td id='name'></td>
            <td id='author'></td>
          </tr>
       </table>
      END
      data = {
       :table1=>[ 
         Lang.new("Ruby", "matz"),
         LangClass.new("Perl", "Larry Wall"),
         Lang.new("Python","Guido van Rossum")
        ]
      }
      tmpl.expand(out="", data)
      formated_out = ""
      Open3.popen3("w3m -T text/html") do |i, o, e|
        i.puts out
        i.close
        formated_out = o.read
      end
      assert_equal(<<END, formated_out)
 name       author
Ruby   matz
Perl   Larry Wall
Python Guido van Rossum

END
    end

    class WebSite < Struct.new(:name, :url)
      def site(m)
        m.site(self.name, :href=>self.url)
      end
    end

    def test_template3
      tmpl = TemplateText.new <<-END
        <table>
          <tr><th>name</th><th>author</th><th>website</th></tr>
          <tr id='table1'>
            <td id='name'></td>
            <td id='author'></td>
            <td id='website'><a id='site'/></td>
          </tr>
       </table>
      END
      data = {
       "table1"=>[ 
         Lang.new("Ruby", "matz", WebSite.new('Ruby home page', 'http://www.ruby-lang.org')),
         Lang.new("Perl", "Larry Wall"),
         { 'name'=>"Python",'author'=>"Guido van Rossum" ,
           'website'=> WebSite.new('Python Programing Language', 'http://www.python.org/')},
        ]
      }
      tmpl.expand(out="", data)
      #print out
      formated_out = ""
      Open3.popen3("w3m -T text/html") do |i, o, e|
        i.puts out
        i.close
        formated_out = o.read
      end
      #print formated_out
      assert_equal(<<END, formated_out)
 name       author               website
Ruby   matz             Ruby home page
Perl   Larry Wall
Python Guido van Rossum Python Programing Language

END
    end
  end

  class TestOption < Test::Unit::TestCase
    include Amrita2

    def test_keep_id
      t = TemplateText.new "<a id='aaa'><b id='bbb'></b></a>" 
      t.keep_id = true
      t.expand(out="") do |m|
        m.aaa do |m2|
          m2.bbb('xxx')
        end
      end
      assert_equal("<a id='aaa'><b id='bbb'>xxx</b></a>", out)
    end

    def test_amrita_id
      t = TemplateText.new "<a aid='aaa'><b aid='bbb'></b></a>" 
      t.amrita_id = :aid
      t.expand(out="") do |m|
        m.aaa do |m2|
          m2.bbb('xxx')
        end
      end
      assert_equal("<a><b>xxx</b></a>", out)
    end

    def test_amrita_id2
      t = TemplateText.new "<a aid='aaa'><b aid='bbb'></b></a>" 
      t.amrita_id = :aid
      t.keep_id = true
      t.expand(out="") do |m|
        m.aaa do |m2|
          m2.bbb('xxx')
        end
      end
      assert_equal("<a aid='aaa'><b aid='bbb'>xxx</b></a>", out)
    end
  end
end

if __FILE__ == $0
  include Amrita2TemplateTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestExpander.suite
    suite << TestOption.suite
    suite << TestTemplate.suite
    suite << TestDefaultExpand.suite
    suite << TestDefaultSpec.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end
